/* ----
 * ---- file   : dsp_config.h
 * ---- author : Bastian Spiegel <bs@tkscript.de>
 * ---- legal  : (c) 2013 by Bastian Spiegel. 
 * ----          Distributed under terms of the LESSER GNU GENERAL PUBLIC LICENSE (LGPL). See 
 * ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING_LGPL for further information.
 * ----
 * ---- info   : Common DSP types and structs. This is part of the "c64_tools" package.
 * ----
 * ---- changed: 06Sep2013, 08Sep2013, 18Sep2013, 03Oct2013, 16Oct2013, 18Oct2013, 19Oct2013
 * ----          21Oct2013
 * ----
 * ----
 */

#ifndef __C64_TOOLS_DSP_CONFIG_H__
#define __C64_TOOLS_DSP_CONFIG_H__

#include "cplusplus_begin.h"


#define C64_IMAGE_SITE_PATH  "/lib/dsp/"
/*#define C64_IMAGE_SITE_PATH  "/lib/c64_tools/images/"*/


/* (note) in order to change the max. size of the DSP image, the kernel needs to be updated as well
 *         so the CMA will return a memory block at a fixed physical address (dsp_config.reset_vector.phys_addr)
 */
#define DSP_MAX_IMAGE_SIZE  (4u * 1024u * 1024u)


/* Only relevant if static memory configuration is used ("USE_CMA_FOR_POOLS" undefined in "kmod/dev.c") 
 *
 *  The total size must not exceed (32MB - DSP_MAX_IMAGE_SIZE)
 */
#define DSP_STATICMEM_CACHE_NONE_SIZE  ( 8u * 1024u * 1024u)
#define DSP_STATICMEM_CACHE_RW_SIZE    (10u * 1024u * 1024u)
#define DSP_STATICMEM_CACHE_R_SIZE     (10u * 1024u * 1024u)


/* L1DSRAM userspace partitioning (24k) */
#define DSP_FSHM_SCRATCH_SIZE  (16u * 1024u)

/* Granularity of dynamic fshm allocations. */
#define DSP_FSHM_ALLOC_GRANULARITY  (64u)
#define DSP_FSHM_ALLOC_GRANULARITY_SHIFT  (6u)

/* Aligned "to-dsp" / "to-gpp" fastcall register size */
#define DSP_FSHM_TOTAL_REG_SIZE  ( ( ((2u * DSP_FSHM_REG_SIZE) + (DSP_FSHM_ALLOC_GRANULARITY - 1u)) >> DSP_FSHM_ALLOC_GRANULARITY_SHIFT ) << DSP_FSHM_ALLOC_GRANULARITY_SHIFT )


/* Maximum number of dynamically allocated fshm regions (default=31) */
#define DSP_FSHM_MAX_DYNAMIC_ALLOCATIONS  ((DSP_FSHM_SIZE - DSP_FSHM_SCRATCH_SIZE - DSP_FSHM_TOTAL_REG_SIZE) / DSP_FSHM_ALLOC_GRANULARITY)
 

#define DSP_RESET_VECTOR_CODESIZE  (0x00000080u)  // size of RESET_VECTOR memory region used for boot code
                                                  //  (actually only 32 bytes are used)
#define DSP_RESET_VECTOR_MINSIZE   (0x00001000u)  // minimum size of RESET_VECTOR memory region

////#define DSP_RESET_VECTOR_MSGSIZE   (DSP_RESET_VECTOR_MINSIZE - DSP_RESET_VECTOR_CODESIZE)


#define DSP_PAGESIZE (4096u)  // (todo) use sysconf(_SC_PAGE_SIZE) [linux]

#define DSP_PAGESIZE_ROUND(a) ( ( ((a)+(DSP_PAGESIZE-1)) / DSP_PAGESIZE ) * DSP_PAGESIZE )

#define DSP_PAGESIZE_ALIGN(a) ( ( (a) / DSP_PAGESIZE ) * DSP_PAGESIZE )



#define DSP_FASTCALL_TIMEOUT  (0x01000000u)


/* printf() ringbuffer */
#define DSP_LOGBUF_BASE_GPP  (0x86002000u)  /* must match MLB_LOGBUF memory in link_core.cmd */
#define DSP_LOGBUF_SIZE      (16u * 1024u)

/* Maximum size of a single ringbuffer log entry */
#define DSP_LOGBUF_LINE_SIZE (512u)


typedef struct {
   dsp_mem_region_t reset_vector;  // contains just a jmp to boot.o64P in .sysinit section (code generated by GPP)
                                   //  must be aligned to 4k page boundary
   dsp_mem_region_t ram;           // DSP image code/data and DSP-side heap
} dsp_config_t;


// (note) config MUST match DSPImage memory config (see <prj>.tci) or hell might break loose
// (note) see "dsp_config.c"
S_EXTERN dsp_config_t dsp_config;


#include "cplusplus_end.h"

#endif /* __C64_TOOLS_DSP_CONFIG_H__ */
